@@ -52,6 +52,9 @@ struct logfmt_json_info {
 static int logfmt_json_keycmp(const void *k1, size_t ksz1, const void *k2,
   size_t ksz2) {
 
+  (void) ksz1;
+  (void) ksz2;
+
   /* Return zero to indicate a match, non-zero otherwise. */
   return (*((unsigned char *) k1) == *((unsigned char *) k2) ? 0 : 1);
 }
@@ -61,6 +64,8 @@ static unsigned int logfmt_json_keyhash(const void *k, size_t ksz) {
   unsigned char c;
   unsigned int res;
 
+  (void) ksz;
+
   c = *((unsigned char *) k);
   res = (c << 8);
 
@@ -516,6 +521,20 @@ int pr_jot_on_json(pool *p, pr_jot_ctx_t *ctx, unsigned char logfmt_id,
   return res;
 }
 
+static int is_data_xfer_cmd(cmd_rec *cmd) {
+  if (pr_cmd_cmp(cmd, PR_CMD_APPE_ID) == 0 ||
+      pr_cmd_cmp(cmd, PR_CMD_LIST_ID) == 0 ||
+      pr_cmd_cmp(cmd, PR_CMD_MLSD_ID) == 0 ||
+      pr_cmd_cmp(cmd, PR_CMD_NLST_ID) == 0 ||
+      pr_cmd_cmp(cmd, PR_CMD_RETR_ID) == 0 ||
+      pr_cmd_cmp(cmd, PR_CMD_STOR_ID) == 0 ||
+      pr_cmd_cmp(cmd, PR_CMD_STOU_ID) == 0) {
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 static char *get_meta_arg(pool *p, unsigned char *meta, size_t *arg_len) {
   char buf[PR_TUNABLE_PATH_MAX+1], *ptr;
   size_t len;
@@ -580,9 +599,10 @@ static const char *get_meta_basename(cmd_rec *cmd) {
     }
 
   } else if (pr_cmd_cmp(cmd, PR_CMD_SITE_ID) == 0 &&
-             (strncasecmp(cmd->argv[1], "CHGRP", 6) == 0 ||
-              strncasecmp(cmd->argv[1], "CHMOD", 6) == 0 ||
-              strncasecmp(cmd->argv[1], "UTIME", 6) == 0)) {
+             cmd->argc > 3 &&
+             (strcasecmp(cmd->argv[1], "CHGRP") == 0 ||
+              strcasecmp(cmd->argv[1], "CHMOD") == 0 ||
+              strcasecmp(cmd->argv[1], "UTIME") == 0)) {
     register unsigned int i;
     char *ptr = "";
 
@@ -648,18 +668,20 @@ static const char *get_meta_dir_name(cmd_rec *cmd) {
   if (pr_cmd_cmp(cmd, PR_CMD_CDUP_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_CWD_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_LIST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_MLSD_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_MKD_ID) == 0 ||
+      pr_cmd_cmp(cmd, PR_CMD_MLSD_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_NLST_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_RMD_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_XCWD_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_XCUP_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_XMKD_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_XRMD_ID) == 0) {
-    char *path, *ptr;
+    char *path, *ptr = NULL;
 
     path = pr_fs_decode_path(p, cmd->arg);
-    ptr = strrchr(path, '/');
+    if (path != NULL) {
+      ptr = strrchr(path, '/');
+    }
 
     if (ptr != NULL) {
       if (ptr != path) {
@@ -691,8 +713,8 @@ static const char *get_meta_dir_path(cmd_rec *cmd) {
 
   if (pr_cmd_cmp(cmd, PR_CMD_CDUP_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_LIST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_MLSD_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_MKD_ID) == 0 ||
+      pr_cmd_cmp(cmd, PR_CMD_MLSD_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_NLST_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_RMD_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_XCUP_ID) == 0 ||
@@ -712,7 +734,7 @@ static const char *get_meta_dir_path(cmd_rec *cmd) {
 
     if (session.chroot_path != NULL) {
       /* Chrooted session. */
-      if (strncmp(pr_fs_getvwd(), "/", 2) == 0) {
+      if (strcmp(pr_fs_getvwd(), "/") == 0) {
         dir_path = session.chroot_path;
 
       } else {
@@ -775,7 +797,7 @@ static const char *get_meta_filename(cmd_rec *cmd) {
      */
     if (session.chroot_path != NULL) {
       /* Chrooted session. */
-      if (strncmp(pr_fs_getvwd(), "/", 2) == 0) {
+      if (strcmp(pr_fs_getvwd(), "/") == 0) {
         filename = session.chroot_path;
 
       } else {
@@ -788,9 +810,10 @@ static const char *get_meta_filename(cmd_rec *cmd) {
     }
 
   } else if (pr_cmd_cmp(cmd, PR_CMD_SITE_ID) == 0 &&
-             (strncasecmp(cmd->argv[1], "CHGRP", 6) == 0 ||
-              strncasecmp(cmd->argv[1], "CHMOD", 6) == 0 ||
-              strncasecmp(cmd->argv[1], "UTIME", 6) == 0)) {
+             cmd->argc > 3 &&
+             (strcasecmp(cmd->argv[1], "CHGRP") == 0 ||
+              strcasecmp(cmd->argv[1], "CHMOD") == 0 ||
+              strcasecmp(cmd->argv[1], "UTIME") == 0)) {
     register unsigned int i;
     char *ptr = "";
 
@@ -854,19 +877,14 @@ static const char *get_meta_transfer_failure(cmd_rec *cmd) {
   /* If the current command is one that incurs a data transfer, then we
    * need to do more work.  If not, it's an easy substitution.
    */
-  if (pr_cmd_cmp(cmd, PR_CMD_APPE_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_LIST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_MLSD_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_NLST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_RETR_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_STOR_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_STOU_ID) == 0) {
+  if (is_data_xfer_cmd(cmd) == TRUE) {
     const char *proto;
 
     proto = pr_session_get_protocol(0);
 
-    if (strncmp(proto, "ftp", 4) == 0 ||
-        strncmp(proto, "ftps", 5) == 0) {
+    if (strcmp(proto, "ftp") == 0 ||
+        strcmp(proto, "ftps") == 0 ||
+        strcmp(proto, "sftp") == 0) {
 
       if (!(XFER_ABORTED)) {
         int res;
@@ -891,7 +909,13 @@ static const char *get_meta_transfer_failure(cmd_rec *cmd) {
               transfer_failure = resp_msg;
             }
           }
+
+        } else {
+          transfer_failure = "unknown";
         }
+
+      } else {
+        transfer_failure = "aborted";
       }
     }
   }
@@ -957,20 +981,14 @@ static const char *get_meta_transfer_status(cmd_rec *cmd) {
   /* If the current command is one that incurs a data transfer, then we need
    * to do more work.  If not, it's an easy substitution.
    */
-  if (pr_cmd_cmp(cmd, PR_CMD_ABOR_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_APPE_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_LIST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_MLSD_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_NLST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_RETR_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_STOR_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_STOU_ID) == 0) {
+  if (is_data_xfer_cmd(cmd) == TRUE ||
+      pr_cmd_cmp(cmd, PR_CMD_ABOR_ID) == 0) {
     const char *proto;
 
     proto = pr_session_get_protocol(0);
 
-    if (strncmp(proto, "ftp", 4) == 0 ||
-        strncmp(proto, "ftps", 5) == 0) {
+    if (strcmp(proto, "ftp") == 0 ||
+        strcmp(proto, "ftps") == 0) {
       if (!(XFER_ABORTED)) {
         int res;
         const char *resp_code = NULL, *resp_msg = NULL;
@@ -1039,13 +1057,7 @@ static int get_meta_transfer_port(cmd_rec *cmd) {
       pr_cmd_cmp(cmd, PR_CMD_PORT_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_EPRT_ID) == 0 ||
       pr_cmd_cmp(cmd, PR_CMD_EPSV_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_APPE_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_LIST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_MLSD_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_NLST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_RETR_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_STOR_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_STOU_ID) == 0) {
+      is_data_xfer_cmd(cmd) == TRUE) {
     transfer_port = session.data_port;
   }
 
@@ -1058,19 +1070,13 @@ static const char *get_meta_transfer_type(cmd_rec *cmd) {
   /* If the current command is one that incurs a data transfer, then we
    * need to do more work.  If not, it's an easy substitution.
    */
-  if (pr_cmd_cmp(cmd, PR_CMD_APPE_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_LIST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_MLSD_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_NLST_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_RETR_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_STOR_ID) == 0 ||
-      pr_cmd_cmp(cmd, PR_CMD_STOU_ID) == 0) {
+  if (is_data_xfer_cmd(cmd) == TRUE) {
     const char *proto;
 
     proto = pr_session_get_protocol(0);
 
-    if (strncmp(proto, "sftp", 5) == 0 ||
-        strncmp(proto, "scp", 4) == 0) {
+    if (strcmp(proto, "sftp") == 0 ||
+        strcmp(proto, "scp") == 0) {
 
       /* Always binary. */
       transfer_type = "binary";
@@ -1491,7 +1497,7 @@ static int resolve_logfmt_id(pool *p, unsigned char logfmt_id,
           method = cmd->argv[0];
         }
 
-      } else {
+      } else if (cmd->argc > 1) {
         char buf[128], *ch;
         size_t len;
 
@@ -1665,7 +1671,7 @@ static int resolve_logfmt_id(pool *p, unsigned char logfmt_id,
 
       val = pr_table_get(cmd->notes, "mod_xfer.file-modified", NULL);
       if (val != NULL) {
-        if (strncmp(val, "true", 5) == 0) {
+        if (strcasecmp(val, "true") == 0) {
           modified = TRUE;
         }
       }
@@ -3045,6 +3051,8 @@ static array_header *filter_get_cmd_ids(pool *p, array_header *names,
   register unsigned int i;
   array_header *cmd_ids;
 
+  (void) excluded_classes;
+
   cmd_ids = make_array(p, names->nelts, sizeof(int));
   for (i = 0; i < names->nelts; i++) {
     const char *name;
